----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    20:42:00 07/15/2014 
-- Design Name: 
-- Module Name:    COMMAND_RECEIVER - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


entity COMMAND_RECEIVER is
    Port ( 
		-- Parameters for the header
		ChannelMask     			: out  std_logic_vector(127 downto 0);
		ADCResolution   			: out  std_logic_vector(3 downto 0);
		
		-- We also need a variable clock that is looking on this SampleRate
		-- 4MHz = 10KHz 
		SampleRate      			: out  std_logic_vector(7 downto 0);
		
		Filter          			: out  std_logic_vector(3 downto 0);
		DeviceID        			: out  std_logic_vector(7 downto 0);		
		
		-- Bitmask for the data selection from the 16 bit samples.
		BITMASK_SAMPLE				: out  std_logic_vector(15 downto 0);			 

		-- Data from the Zarlink: 
		FIFO_OUT_DATA   			: in STD_LOGIC_VECTOR(7 downto 0);
		FIFO_OUT_VALID   			: in STD_LOGIC;
	 
		-- RHA TEST MODE
		TestMode					: out STD_LOGIC;
		
		-- RHA clock 
		Clk_Reduced					: out  STD_LOGIC;
		Clock_4MHz					: out  STD_LOGIC;
		ResetRHAandFIFO				: out STD_LOGIC;
		Clk_FAST					: in  STD_LOGIC;
		Clk 						: in  STD_LOGIC;
		Reset 						: in  STD_LOGIC
		);
end COMMAND_RECEIVER;

architecture Behavioral of COMMAND_RECEIVER is

	signal ChannelMask_INTERNAL     			: std_logic_vector(127 downto 0) := (others => '0');
	signal ADCResolution_INTERNAL   			: std_logic_vector(3 downto 0) := (others => '0');
	signal ADCResolution_TOP_INTERNAL  		: std_logic_vector(3 downto 0) := (others => '0');
	signal SampleRate_INTERNAL      			: std_logic_vector(7 downto 0) := (others => '0');
	signal Filter_INTERNAL          			: std_logic_vector(3 downto 0) := (others => '0');
	signal BITMASK_SAMPLE_INTERNAL			: std_logic_vector(15 downto 0) := (others => '0');			 
	signal BITMASK_SAMPLE_TOP_INTERNAL		: std_logic_vector(15 downto 0) := (others => '0');
	signal ResetRHAandFIFO_INTERNAL			: STD_LOGIC := '0';

	constant HeaderByte_1	: std_logic_vector(7 downto 0) := "00110000"; -- 48
	constant HeaderByte_2	: std_logic_vector(7 downto 0) := "01101001"; -- 105
	constant Implant_ID     : std_logic_vector(7 downto 0) := "00110101"; -- Implant ID 53
	constant Command_Enable : std_logic_vector(7 downto 0) := "01100101"; -- 101 e
	constant Command_Disable: std_logic_vector(7 downto 0) := "01100100"; -- 100 d
	constant Command_Channel: std_logic_vector(7 downto 0) := "01100011"; --  99 c
	constant Command_Filter : std_logic_vector(7 downto 0) := "01100110"; -- 102 f
	constant Command_SRate  : std_logic_vector(7 downto 0) := "01110011"; -- 115 s
	constant Command_ADCReso: std_logic_vector(7 downto 0) := "01110010"; -- 114 r
	constant Command_ADCResT: std_logic_vector(7 downto 0) := "01110100"; -- 116 t
	constant Command_TestMode_ON: std_logic_vector(7 downto 0) := "01110101"; -- u
	constant Command_TestMode_OFF: std_logic_vector(7 downto 0) := "01110110"; -- v
	
--	signal Clk_6MHz : std_logic := '0';

	type STATE_Type is (
		SM_IDLE,
		SM_HEADER_CHECK_A,
		SM_HEADER_CHECK_B,
		SM_PROCESS,
		SM_COMMAND_FILTER,
		SM_COMMAND_SRATE,
		SM_COMMAND_ADCRES,
		SM_COMMAND_ADCREST,
		SM_COMMAND_CHANNEL_01,
		SM_COMMAND_CHANNEL_02,
		SM_COMMAND_CHANNEL_03,
		SM_COMMAND_CHANNEL_04,
		SM_COMMAND_CHANNEL_05,
		SM_COMMAND_CHANNEL_06,
		SM_COMMAND_CHANNEL_07,
		SM_COMMAND_CHANNEL_08,
		SM_COMMAND_CHANNEL_09,
		SM_COMMAND_CHANNEL_10,
		SM_COMMAND_CHANNEL_11,
		SM_COMMAND_CHANNEL_12,
		SM_COMMAND_CHANNEL_13,
		SM_COMMAND_CHANNEL_14,		
		SM_COMMAND_CHANNEL_15,		
		SM_COMMAND_CHANNEL_16
	);	
	signal SPIMACHINE_STATE : STATE_TYPE;
	
	signal Clk_Reduced_Internal : std_logic := '0';
	signal Counter : unsigned(7 downto 0) := (others => '0'); 
	signal Counter_4MHz : unsigned(1 downto 0) := (others => '0');
	signal FIFO_OUT_VALID_LAST : STD_LOGIC := '0';
	
	signal Clk_4MHz						: STD_LOGIC := '0';
	
	signal TestMode_INTERAL				: STD_LOGIC := '0';

begin
	TestMode <= TestMode_INTERAL;
	
	-- Export internal buffered signals 
	ChannelMask <= ChannelMask_INTERNAL;
	ADCResolution <= ADCResolution_INTERNAL;
	SampleRate <= SampleRate_INTERNAL;
	Filter <= Filter_INTERNAL;
	DeviceID <= Implant_ID;
	Clock_4MHz <= Clk_4MHz;
	
	-- Combine the upper and the lower bitmask for the samples 
	BITMASK_SAMPLE( 0) <= BITMASK_SAMPLE_INTERNAL( 0) AND BITMASK_SAMPLE_TOP_INTERNAL( 0);
	BITMASK_SAMPLE( 1) <= BITMASK_SAMPLE_INTERNAL( 1) AND BITMASK_SAMPLE_TOP_INTERNAL( 1);
	BITMASK_SAMPLE( 2) <= BITMASK_SAMPLE_INTERNAL( 2) AND BITMASK_SAMPLE_TOP_INTERNAL( 2);
	BITMASK_SAMPLE( 3) <= BITMASK_SAMPLE_INTERNAL( 3) AND BITMASK_SAMPLE_TOP_INTERNAL( 3);
	BITMASK_SAMPLE( 4) <= BITMASK_SAMPLE_INTERNAL( 4) AND BITMASK_SAMPLE_TOP_INTERNAL( 4);
	BITMASK_SAMPLE( 5) <= BITMASK_SAMPLE_INTERNAL( 5) AND BITMASK_SAMPLE_TOP_INTERNAL( 5);
	BITMASK_SAMPLE( 6) <= BITMASK_SAMPLE_INTERNAL( 6) AND BITMASK_SAMPLE_TOP_INTERNAL( 6);
	BITMASK_SAMPLE( 7) <= BITMASK_SAMPLE_INTERNAL( 7) AND BITMASK_SAMPLE_TOP_INTERNAL( 7);
	BITMASK_SAMPLE( 8) <= BITMASK_SAMPLE_INTERNAL( 8) AND BITMASK_SAMPLE_TOP_INTERNAL( 8);
	BITMASK_SAMPLE( 9) <= BITMASK_SAMPLE_INTERNAL( 9) AND BITMASK_SAMPLE_TOP_INTERNAL( 9);
	BITMASK_SAMPLE(10) <= BITMASK_SAMPLE_INTERNAL(10) AND BITMASK_SAMPLE_TOP_INTERNAL(10);
	BITMASK_SAMPLE(11) <= BITMASK_SAMPLE_INTERNAL(11) AND BITMASK_SAMPLE_TOP_INTERNAL(11);
	BITMASK_SAMPLE(12) <= BITMASK_SAMPLE_INTERNAL(12) AND BITMASK_SAMPLE_TOP_INTERNAL(12);
	BITMASK_SAMPLE(13) <= BITMASK_SAMPLE_INTERNAL(13) AND BITMASK_SAMPLE_TOP_INTERNAL(13);
	BITMASK_SAMPLE(14) <= BITMASK_SAMPLE_INTERNAL(14) AND BITMASK_SAMPLE_TOP_INTERNAL(14);
	BITMASK_SAMPLE(15) <= BITMASK_SAMPLE_INTERNAL(15) AND BITMASK_SAMPLE_TOP_INTERNAL(15);
	

	ResetRHAandFIFO <= ResetRHAandFIFO_INTERNAL;
	
	
	CMD_RECEIVER : process (Clk,Reset)
	begin
		if Reset = '1' then
			ChannelMask_INTERNAL <= (others => '0');
			ADCResolution_INTERNAL <= (others => '1');
			ADCResolution_TOP_INTERNAL <= (others => '0');
			SampleRate_INTERNAL <= (others => '0');
			Filter_INTERNAL <= (others => '0');
			ResetRHAandFIFO_INTERNAL <= '1';
			
			SPIMACHINE_STATE <= SM_IDLE;
			FIFO_OUT_VALID_LAST <= '0';
			TestMode_INTERAL <= '1';
			
		elsif rising_edge(Clk) then
			ChannelMask_INTERNAL <= ChannelMask_INTERNAL;
			ADCResolution_INTERNAL <= ADCResolution_INTERNAL;
			ADCResolution_TOP_INTERNAL <= ADCResolution_TOP_INTERNAL;
			SampleRate_INTERNAL <= SampleRate_INTERNAL;
			Filter_INTERNAL <= Filter_INTERNAL;
			TestMode_INTERAL <= TestMode_INTERAL;
			ResetRHAandFIFO_INTERNAL <= ResetRHAandFIFO_INTERNAL;
			FIFO_OUT_VALID_LAST <= FIFO_OUT_VALID;
			-- Failsafe !
			SPIMACHINE_STATE <= SM_IDLE;

			case SPIMACHINE_STATE is
				when SM_IDLE =>
					SPIMACHINE_STATE <= SM_IDLE;
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = HeaderByte_1) then

						SPIMACHINE_STATE <= SM_HEADER_CHECK_A;
					else
						null;
					end if;

				when SM_HEADER_CHECK_A =>
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = HeaderByte_2) then
						SPIMACHINE_STATE <= SM_HEADER_CHECK_B;
					elsif (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_IDLE;
					else
						SPIMACHINE_STATE <= SM_HEADER_CHECK_A;
					end if;

				when SM_HEADER_CHECK_B =>
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = Implant_ID) then
						SPIMACHINE_STATE <= SM_PROCESS;
					elsif (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_IDLE;
					else
						SPIMACHINE_STATE <= SM_HEADER_CHECK_B;
					end if;
					
				when SM_PROCESS => 
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = Command_Enable) then
						SPIMACHINE_STATE <= SM_IDLE;
						ResetRHAandFIFO_INTERNAL <= '0';
					elsif (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = Command_Disable) then
						SPIMACHINE_STATE <= SM_IDLE;
						ResetRHAandFIFO_INTERNAL <= '1';
					elsif (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = Command_Channel) then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_01;
						ResetRHAandFIFO_INTERNAL <= '1';
					elsif (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = Command_Filter) then
						SPIMACHINE_STATE <= SM_COMMAND_FILTER;
						ResetRHAandFIFO_INTERNAL <= '1';
					elsif (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = Command_SRate) then
						SPIMACHINE_STATE <= SM_COMMAND_SRATE;
						ResetRHAandFIFO_INTERNAL <= '1';
					elsif (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = Command_ADCReso) then
						SPIMACHINE_STATE <= SM_COMMAND_ADCRES;
						ResetRHAandFIFO_INTERNAL <= '1';
					elsif (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = Command_ADCResT) then
						SPIMACHINE_STATE <= SM_COMMAND_ADCREST;
						ResetRHAandFIFO_INTERNAL <= '1';
					elsif (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = Command_TestMode_ON) then
						SPIMACHINE_STATE <= SM_IDLE;
						TestMode_INTERAL <= '1';
					elsif (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') AND (FIFO_OUT_DATA = Command_TestMode_OFF) then
						SPIMACHINE_STATE <= SM_IDLE;
						TestMode_INTERAL <= '0';						
					else
						SPIMACHINE_STATE <= SM_PROCESS;
					end if;

				-- Set the RHA Filter settings 
				when SM_COMMAND_FILTER =>
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_IDLE;
						Filter_INTERNAL(3 downto 0) <= FIFO_OUT_DATA(3 downto 0);
					else
						SPIMACHINE_STATE <= SM_COMMAND_FILTER;
					end if;

				-- Set Sample Rate (we need a variable clock that is looking on this)	
				when SM_COMMAND_SRATE =>
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_IDLE;
						SampleRate_INTERNAL(7 downto 0) <= FIFO_OUT_DATA(7 downto 0);
					else
						SPIMACHINE_STATE <= SM_COMMAND_SRATE;
					end if;
				
				-- Set the ADC sample resolution	(Lower Bits)
				when SM_COMMAND_ADCRES =>
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_IDLE;
						ADCResolution_INTERNAL(3 downto 0) <= FIFO_OUT_DATA(3 downto 0);
					else
						SPIMACHINE_STATE <= SM_COMMAND_ADCRES;
					end if;

				-- Set the ADC sample resolution	(Higher Bits)
				when SM_COMMAND_ADCREST =>
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_IDLE;
						ADCResolution_TOP_INTERNAL(3 downto 0) <= FIFO_OUT_DATA(3 downto 0);
					else
						SPIMACHINE_STATE <= SM_COMMAND_ADCREST;
					end if;

		-- CHANNEL MASK						
				when SM_COMMAND_CHANNEL_01 =>
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_02;					
						ChannelMask_INTERNAL(  7 downto   0) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_01;
					end if;
						
				when SM_COMMAND_CHANNEL_02 =>
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_03;					
						ChannelMask_INTERNAL( 15 downto   8) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_02;
					end if;
						
				when SM_COMMAND_CHANNEL_03 =>			
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_04;					
						ChannelMask_INTERNAL( 23 downto  16) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_03;
					end if;
						
				when SM_COMMAND_CHANNEL_04 =>
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_05;					
						ChannelMask_INTERNAL( 31 downto  24) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_04;
					end if;
						
				when SM_COMMAND_CHANNEL_05 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_06;					
						ChannelMask_INTERNAL( 39 downto  32) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_05;
					end if;

				when SM_COMMAND_CHANNEL_06 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_07;					
						ChannelMask_INTERNAL( 47 downto  40) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_06;
					end if;
						
				when SM_COMMAND_CHANNEL_07 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_08;					
						ChannelMask_INTERNAL( 55 downto  48) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_07;
					end if;
						
				when SM_COMMAND_CHANNEL_08 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_09;					
						ChannelMask_INTERNAL( 63 downto  56) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_08;
					end if;
						
				when SM_COMMAND_CHANNEL_09 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_10;					
						ChannelMask_INTERNAL( 71 downto  64) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_09;
					end if;
						
				when SM_COMMAND_CHANNEL_10 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_11;					
						ChannelMask_INTERNAL( 79 downto  72) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_10;
					end if;
						
				when SM_COMMAND_CHANNEL_11 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_12;					
						ChannelMask_INTERNAL( 87 downto  80) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_11;
					end if;
						
				when SM_COMMAND_CHANNEL_12 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_13;					
						ChannelMask_INTERNAL( 95 downto  88) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_12;
					end if;
						
				when SM_COMMAND_CHANNEL_13 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_14;					
						ChannelMask_INTERNAL(103 downto  96) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_13;
					end if;

				when SM_COMMAND_CHANNEL_14 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_15;					
						ChannelMask_INTERNAL(111 downto 104) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_14;
					end if;

				when SM_COMMAND_CHANNEL_15 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_16;					
						ChannelMask_INTERNAL(119 downto 112) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_15;
					end if;
						
				when SM_COMMAND_CHANNEL_16 =>					
					if (FIFO_OUT_VALID = '1') AND (FIFO_OUT_VALID_LAST = '0') then
						SPIMACHINE_STATE <= SM_IDLE;
						ChannelMask_INTERNAL(127 downto 120) <= FIFO_OUT_DATA;
					else
						SPIMACHINE_STATE <= SM_COMMAND_CHANNEL_16;
					end if;
					
				when others =>
					ResetRHAandFIFO_INTERNAL <= '1';
					SPIMACHINE_STATE <= SM_IDLE;
			end case;						
			
		end if;
	end process CMD_RECEIVER;			

	ADC_R : process (Reset,Clk)
	begin
		if Reset = '1' then
			BITMASK_SAMPLE_INTERNAL <= "1111111111111111";
		elsif rising_edge(Clk) then	
			case ADCResolution_INTERNAL is
				when "0000" =>
					BITMASK_SAMPLE_INTERNAL <= "0000000000000001";						
				when "0001" =>
					BITMASK_SAMPLE_INTERNAL <= "0000000000000011";						
				when "0010" =>
					BITMASK_SAMPLE_INTERNAL <= "0000000000000111";
				when "0011" =>
					BITMASK_SAMPLE_INTERNAL <= "0000000000001111";
				when "0100" =>
					BITMASK_SAMPLE_INTERNAL <= "0000000000011111";						
				when "0101" =>
					BITMASK_SAMPLE_INTERNAL <= "0000000000111111";												
				when "0110" =>
					BITMASK_SAMPLE_INTERNAL <= "0000000001111111";						
				when "0111" =>
					BITMASK_SAMPLE_INTERNAL <= "0000000011111111";						
				when "1000" =>
					BITMASK_SAMPLE_INTERNAL <= "0000000111111111";						
				when "1001" =>
					BITMASK_SAMPLE_INTERNAL <= "0000001111111111";						
				when "1010" =>
					BITMASK_SAMPLE_INTERNAL <= "0000011111111111";						
				when "1011" =>
					BITMASK_SAMPLE_INTERNAL <= "0000111111111111";						
				when "1100" =>
					BITMASK_SAMPLE_INTERNAL <= "0001111111111111";						
				when "1101" =>
					BITMASK_SAMPLE_INTERNAL <= "0011111111111111";						
				when "1110" =>
					BITMASK_SAMPLE_INTERNAL <= "0111111111111111";						
				when "1111" =>
					BITMASK_SAMPLE_INTERNAL <= "1111111111111111";						
				when others =>
					BITMASK_SAMPLE_INTERNAL <= "0000000000000001";
			end case;	
		end if;
	end process ADC_R;	
	
	ADC_R_T : process (Reset,Clk)
	begin
		if Reset = '1' then
			BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000000000000";
		elsif rising_edge(Clk) then
			case ADCResolution_TOP_INTERNAL is
				when "0000" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000000000000";						
				when "0001" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000000000001";						
				when "0010" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000000000011";
				when "0011" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000000000111";
				when "0100" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000000001111";						
				when "0101" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000000011111";												
				when "0110" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000000111111";						
				when "0111" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000001111111";						
				when "1000" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000011111111";						
				when "1001" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000111111111";						
				when "1010" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000001111111111";						
				when "1011" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000011111111111";						
				when "1100" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000111111111111";						
				when "1101" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0001111111111111";						
				when "1110" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0011111111111111";						
				when "1111" =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0111111111111111";						
				when others =>
					BITMASK_SAMPLE_TOP_INTERNAL <= NOT "0000000000000001";
			end case;	
		end if;
	end process ADC_R_T;						

	CLK_GATE : process (SampleRate_INTERNAL,Reset,Clk,Clk_Reduced_Internal)
	begin
		if Reset = '1' then
			Clk_Reduced <= '0';
		elsif SampleRate_INTERNAL = "00000000" then
			Clk_Reduced <= Clk;
		else
			Clk_Reduced <= Clk_Reduced_Internal;
		end if;
	end process CLK_GATE;	

	CLK_REDUCE : process (Reset,Clk)
	begin
		if Reset = '1' then
			Counter <= to_unsigned(1,8);
			Clk_Reduced_Internal <= '0';
			
		elsif rising_edge(Clk) then
			Clk_Reduced_Internal <= Clk_Reduced_Internal;
			Counter <= Counter;

			if SampleRate_INTERNAL = "00000000" then
				Clk_Reduced_Internal <= '0';
				Counter <= to_unsigned(1,8);
			elsif std_logic_vector(Counter) = SampleRate_INTERNAL then
				Clk_Reduced_Internal <= NOT Clk_Reduced_Internal;
				Counter <= to_unsigned(1,8);
			else
				Counter <= Counter + 1;
			end if;
			
		end if;
	end process CLK_REDUCE;	


--	CLK_6MHz_PROCESS : process (Reset,Clk)
--	begin
--		if Reset = '1' then
--			Clk_6MHz <= '0';
--		elsif rising_edge(Clk) then
--			Clk_6MHz <= NOT Clk_6MHz;
--		end if;
--	end process CLK_6MHz_PROCESS;	


	-- We reduce the 24 MHz to a 4 MHz clock 
	CLK_4MHZ_PROCESS : process (Reset,Clk_FAST)
	begin
		if Reset = '1' then
			Counter_4MHz <= to_unsigned(0,2);
			Clk_4MHz <= '0';
		elsif rising_edge(Clk_FAST) then
			Clk_4MHz <= Clk_4MHz;
			Counter_4MHz <= Counter_4MHz;
			if std_logic_vector(Counter_4MHz) = "10" then
				Clk_4MHz <= NOT Clk_4MHz;
				Counter_4MHz <= to_unsigned(0,2);
			else
				Counter_4MHz <= Counter_4MHz + 1;
			end if;
			
		end if;
	end process CLK_4MHZ_PROCESS;	


end Behavioral;

