----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    17:49:58 07/23/2014 
-- Design Name: 
-- Module Name:    DATA_ACQUISITION_BLOCK - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity DATA_ACQUISITION_BLOCK is
    Port (
		-- For the data stream
		RHA_ADC_out 						: in  STD_LOGIC_VECTOR(7 downto 0);
		-- Where a RHA data frame starts 
		RHA_sync 							: in  STD_LOGIC_VECTOR(7 downto 0);
		-- RHA ADC clock
		RHA_ADC_clk							: out STD_LOGIC_VECTOR(7 downto 0);

		-- This goes into the Zarlink / FIFO direction: 
		FIFO_OUT_DATA   					: out STD_LOGIC_VECTOR(7 downto 0);
		FIFO_OUT_VALID   					: out STD_LOGIC;
		-- We need the FIFO to have (at least)
		-- 280 x 8 bit (2240 bits)
		-- of free space for the next block to be on the safe side.		
		ExportNext128ChannelBlock		:in STD_LOGIC;

		-- Parameters for the header
		ChannelMask     					: in  std_logic_vector(127 downto 0);
		ADCResolution   					: in  std_logic_vector(3 downto 0);
		SampleRate      					: in  std_logic_vector(7 downto 0);
		Filter          					: in  std_logic_vector(3 downto 0);
		DeviceID        					: in  std_logic_vector(7 downto 0);		
		BITMASK_SAMPLE						: in  std_logic_vector(15 downto 0);

		-- Error flags 
		ERROR_RHA_ASYNC_FIRST_OUT 		: out STD_LOGIC;
		ERROR_RHA_ASYNC_VALID_OUT 		: out STD_LOGIC;
		ERROR_PARITY_OUT 					: out STD_LOGIC;
		ERROR_PATTERN_OUT 				: out STD_LOGIC;
		ERROR_ADDRESS_OUT 				: out STD_LOGIC;
		ERROR_BUFFEROVERFLOW_OUT 		: out STD_LOGIC;	 
		ERROR_ALL_OUT 						: out STD_LOGIC;

		-- RHA Test Mode
		TestMode								: in  std_logic;
		Clk_Slow	        					: in  STD_LOGIC;
     	Reset 		         			: in  STD_LOGIC;	 
		Clk 									: in  STD_LOGIC
	);
end DATA_ACQUISITION_BLOCK;

architecture Behavioral of DATA_ACQUISITION_BLOCK is

        -- The collected output, brocken into single values       
		signal DATAOUT_DATA_OUT				: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DATAOUT_VALID_OUT			: STD_LOGIC := '0';
		signal NewBlock						: STD_LOGIC := '0';
		signal ReadRequest					: STD_LOGIC := '0';
		

		signal RHA_ADC_out_TEST				: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal RHA_sync_TEST					: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal RHA_ADC_clk_TEST				: STD_LOGIC := '0';

		signal RHA_ADC_out_Array 			: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal RHA_sync_Array 				: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal RHA_ADC_clk_Array			: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');

		-- Error Flags 
		signal ERROR_RHA_ASYNC_FIRST 		: std_logic := '0';
		signal ERROR_RHA_ASYNC_VALID 		: std_logic := '0';
		signal ERROR_PARITY 					: std_logic := '0';
		signal ERROR_PATTERN 				: std_logic := '0';
		signal ERROR_ADDRESS 				: std_logic := '0';
		signal ERROR_BUFFEROVERFLOW 		: std_logic := '0';
		
begin

	
	-- This module joins the output all 8 RHAs ICs and has 
	-- sub-modules for handling the SPI connection with the
	-- RHAs. 
	Inst_RHA_ARRAY: entity work.RHA_ARRAY 
	PORT MAP(
		RHA_ADC_out => RHA_ADC_out_Array,
		RHA_sync => RHA_sync_Array,
		RHA_ADC_clk => RHA_ADC_clk_Array,
		RHA_CHANNEL_MASK => ChannelMask,
		DATAOUT_DATA_OUT_EXT => DATAOUT_DATA_OUT,
		DATAOUT_VALID_OUT_EXT => DATAOUT_VALID_OUT,
      	ERROR_RHA_ASYNC_FIRST => ERROR_RHA_ASYNC_FIRST,
      	ERROR_RHA_ASYNC_VALID => ERROR_RHA_ASYNC_VALID,
      	ERROR_PARITY => ERROR_PARITY,
      	ERROR_PATTERN => ERROR_PATTERN,
      	ERROR_ADDRESS => ERROR_ADDRESS,
		ERROR_BUFFEROVERFLOW => ERROR_BUFFEROVERFLOW,		
		NewBlock => NewBlock,
		ReadRequest => ReadRequest,
		Clk_Slow => Clk_Slow,
		Clk => Clk,
		Reset_RHA => Reset,
		Reset => Reset
	);

	Inst_RHA_TO_ZL_CONVERTER: entity work.RHA_TO_ZL_CONVERTER 
	PORT MAP(
		DATAOUT_DATA_OUT => DATAOUT_DATA_OUT,
		DATAOUT_VALID_OUT => DATAOUT_VALID_OUT,
		NewBlock => NewBlock,
		ReadRequest => ReadRequest,
		FIFO_OUT_DATA => FIFO_OUT_DATA,
		FIFO_OUT_VALID => FIFO_OUT_VALID,
		ChannelMask => ChannelMask,
		ADCResolution => ADCResolution,
		SampleRate => SampleRate,
		Filter => Filter,
		DeviceID => DeviceID,
		BITMASK_SAMPLE => BITMASK_SAMPLE,
		ExportNext128ChannelBlock => ExportNext128ChannelBlock,
		Clk => Clk,
		Clk_SLOW => Clk_Slow,
		Reset => Reset
	);

	Inst_RHA_TESTMODULE: entity work.RHA_TESTMODULE 
	PORT MAP(
		RHA_ADC_out => RHA_ADC_out_TEST,
		RHA_ADC_sync => open,
		RHA_sync => RHA_sync_TEST,
		RHA_ADC_clk => Clk_Slow,
		RHA_Reset => Reset 
	);

	RHA_ADC_clk <= RHA_ADC_clk_Array;

	-- RHA TESTMODE
	TestMode_Process : process (TestMode,RHA_sync_TEST,RHA_ADC_out_TEST,RHA_sync,RHA_ADC_out)
	begin	
		if TestMode = '1' then
			RHA_sync_Array <= RHA_sync_TEST;
			RHA_ADC_out_Array <= RHA_ADC_out_TEST;
		else
			RHA_sync_Array <= RHA_sync;
			RHA_ADC_out_Array <= RHA_ADC_out;
		end if;
	end process TestMode_Process;

	-- Checking every used RHA sample for errors 
	RHA_ERROR_WATCHDOG : entity work.RHA_ERROR_WATCHDOG 
	Port MAP( 
		ERROR_RHA_ASYNC_FIRST => ERROR_RHA_ASYNC_FIRST,
		ERROR_RHA_ASYNC_VALID => ERROR_RHA_ASYNC_VALID,
		ERROR_PARITY => ERROR_PARITY,
		ERROR_PATTERN => ERROR_PATTERN,
		ERROR_ADDRESS => ERROR_ADDRESS,
		ERROR_BUFFEROVERFLOW	=> ERROR_BUFFEROVERFLOW,
		ERROR_RHA_ASYNC_FIRST_OUT => ERROR_RHA_ASYNC_FIRST_OUT,
		ERROR_RHA_ASYNC_VALID_OUT => ERROR_RHA_ASYNC_VALID_OUT,
		ERROR_PARITY_OUT => ERROR_PARITY_OUT,
		ERROR_PATTERN_OUT => ERROR_PATTERN_OUT,
		ERROR_ADDRESS_OUT => ERROR_ADDRESS_OUT,
		ERROR_BUFFEROVERFLOW_OUT => ERROR_BUFFEROVERFLOW_OUT,
		ERROR_ALL_OUT => ERROR_ALL_OUT,
		DATAOUT_VALID_OUT => DATAOUT_VALID_OUT,
		Clk => Clk,  	
      Reset => Reset	 
	);
	
end architecture Behavioral;

