----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    18:14:56 07/23/2014 
-- Design Name: 
-- Module Name:    IMPLANT_TOP - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;


entity IMPLANT_TOP is
    Port ( 
	 		-- Connection to the Zarlink 
		SPI_ZL_CS 						: out STD_LOGIC;
		SPI_SDI 						: out STD_LOGIC;
		SPI_SDO 						: in  STD_LOGIC;
		SPI_CLK 						: out STD_LOGIC;		

		-- For the data stream
		RHA_ADC_out 					: in  STD_LOGIC_VECTOR(7 downto 0);
		-- Where a RHA data frame starts 
		RHA_sync 						: in  STD_LOGIC_VECTOR(7 downto 0);
		-- RHA ADC clock
		RHA_ADC_clk						: out STD_LOGIC_VECTOR(7 downto 0);

		-- RHA Reset
		RHA_Reset						: out STD_LOGIC_VECTOR(7 downto 0);
	
		-- Debugging the Zarlink 
		DEBUG_ZARLINK_DATA				: out STD_LOGIC_VECTOR(15 downto 0);
		DEBUG_ZARLINK_VALID				: out STD_LOGIC;

		-- Error flags 
		ERROR_RHA_ASYNC_FIRST_OUT 		: out STD_LOGIC;
		ERROR_RHA_ASYNC_VALID_OUT 		: out STD_LOGIC;
		ERROR_PARITY_OUT 				: out STD_LOGIC;
		ERROR_PATTERN_OUT 				: out STD_LOGIC;
		ERROR_ADDRESS_OUT 				: out STD_LOGIC;
		ERROR_BUFFEROVERFLOW_OUT 		: out STD_LOGIC;	 
		ERROR_ALL_OUT 					: out STD_LOGIC;

		DEBUG_STRING_ORGANIZER			: out std_logic_vector (3 downto 0);	

		-- The usual ports 
		Reset 							: in  STD_LOGIC;				
		Clk_FAST 						: in  STD_LOGIC

	 );
end IMPLANT_TOP;

architecture Behavioral of IMPLANT_TOP is

		-- RHA 
		signal ResetRHAandFIFO				: STD_LOGIC := '0';

		-- Parameters for the header
		signal ChannelMask     				: std_logic_vector(127 downto 0) := (others => '0');
		signal ADCResolution   				: std_logic_vector(3 downto 0) := (others => '0');
		
		-- We also need a variable clock that is looking on this SampleRate
		-- 4MHz = 10KHz 
		signal SampleRate      				: std_logic_vector(7 downto 0) := (others => '0');
		
		signal Filter          				: std_logic_vector(3 downto 0) := (others => '0');
		signal DeviceID        				: std_logic_vector(7 downto 0) := (others => '0');		
		
		-- Bitmask for the data selection from the 16 bit samples.
		signal BITMASK_SAMPLE				: std_logic_vector(15 downto 0) := (others => '0');			 
		signal Clk_Reduced					: STD_LOGIC := '0';

		-- Connection to the RHA DATA FIFO...
		signal FIFO_IN_DATA					: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');	
		signal FIFO_IN_VALID					: STD_LOGIC := '0';
		signal FIFO_IN_READENABLE			: STD_LOGIC := '0';

		-- This goes into the Zarlink / FIFO direction: 
		signal FIFO_OUT_DATA   				: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal FIFO_OUT_VALID   			: STD_LOGIC := '0';
		-- We need the FIFO to have (at least)
		-- 21 x 112 bit (2352 its)
		-- of free space for the next block to be on the safe side.		
		signal ExportNext128ChannelBlock	: STD_LOGIC := '0';

		-- FIFO for XILINX 
		-- prog_full = 3800 (we need at least 280 x 8bit space for a new block)
		-- worst case: 28x 8 bit header + 256 x 8 bit data 
		-- prog_empty = 13 (14 equals not empty)
		-- in and out are 8 bit wide
		-- space for 4096x 8bit
		-- different clocks are not neccessary 
		-- but the standard IGLOO FIFO has it
		-- -> let's doit in a similar fashion
		COMPONENT FIFO
		PORT (
			rst : IN STD_LOGIC;
			wr_clk : IN STD_LOGIC;
			rd_clk : IN STD_LOGIC;
			din : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
			wr_en : IN STD_LOGIC;
			rd_en : IN STD_LOGIC;
			dout : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
			full : OUT STD_LOGIC;
			empty : OUT STD_LOGIC;
			prog_full : OUT STD_LOGIC;
			prog_empty : OUT STD_LOGIC
		);
		END COMPONENT;
	
		signal FIFO_AFULL : STD_LOGIC := '0';
		signal FIFO_AEMPTY : STD_LOGIC := '0';
		signal FIFO_EMPTY : STD_LOGIC := '0';
		signal FIFO_FULL : STD_LOGIC := '0';
		-- Domain Crossing! 
		signal FIFO_IN_READENABLE_DX : STD_LOGIC := '0';
		signal FIFO_IN_READENABLE_LAST  : STD_LOGIC := '0';
		signal dout : STD_LOGIC_VECTOR(7 DOWNTO 0) := (others => '0');
		signal ResetRHAandFIFO_DELAYED : STD_LOGIC := '0'; 
		signal RHA_RESET_COUNTER : unsigned (4 downto 0) := (others => '0');

		-- Decouple 
		signal din_DECOUPLE  : STD_LOGIC_VECTOR(7 DOWNTO 0) := (others => '0');
		signal wr_en_DECOUPLE  : STD_LOGIC := '0';
		signal rd_en_DECOUPLE  : STD_LOGIC := '0';
		signal dout_DECOUPLE  : STD_LOGIC_VECTOR(7 DOWNTO 0) := (others => '0');
		signal full_DECOUPLE  : STD_LOGIC := '0';
		signal empty_DECOUPLE  : STD_LOGIC := '0';
		signal prog_full_DECOUPLE  : STD_LOGIC := '0';
		signal prog_empty_DECOUPLE  : STD_LOGIC := '0';
		

		-- Test 
		signal Counter_XXX 	: unsigned(6 downto 0) := (others => '0');

		-- DEBUG ZARLINK COMMUNICATION
		signal DEBUG_ZARLINK_DATA_INTERNAL		: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DEBUG_ZARLINK_VALID_INTERNAL		: STD_LOGIC := '0';
		signal DEBUG_COUNTER							: unsigned (3 downto 0) := (others => '0');
	
		signal SPI_ZL_CS_BUFFER	: STD_LOGIC := '0';
		signal SPI_SDI_BUFFER	: STD_LOGIC := '0';
		signal SPI_CLK_BUFFER	: STD_LOGIC := '0';
		signal Clk		 		: STD_LOGIC := '0';
		-- RHA TEST MODE
		signal TestMode 		: STD_LOGIC := '0';
begin
	-- DEBUG ZARLINK COMMUNICATION
	DEBUG_ZARLINK_DATA <= DEBUG_ZARLINK_DATA_INTERNAL;
	DEBUG_ZARLINK_VALID <= DEBUG_ZARLINK_VALID_INTERNAL;

	SPI_ZL_CS <= SPI_ZL_CS_BUFFER;
	SPI_SDI <= SPI_SDI_BUFFER;
	SPI_CLK <= SPI_CLK_BUFFER;

	-- Xilinx FIFO Version 
	MyFIFO : FIFO
	PORT MAP (
		--rst => Reset,
		rst => ResetRHAandFIFO,
		wr_clk => Clk,
		rd_clk => Clk,
		din => din_DECOUPLE,
		wr_en => wr_en_DECOUPLE,
		rd_en => rd_en_DECOUPLE,
		dout => dout_DECOUPLE,
		full => full_DECOUPLE,
		empty => empty_DECOUPLE,
		prog_full => prog_full_DECOUPLE, 
		prog_empty => prog_empty_DECOUPLE
	);

	DECOUPLE_FIFO : process (Clk,Reset)
	begin
		if Reset = '1' then
			-- Reset 
			din_DECOUPLE <= (others => '0');
			wr_en_DECOUPLE <= '0';

			dout <= (others => '0');
			rd_en_DECOUPLE <= '0';

			FIFO_AFULL <= '1';
			FIFO_FULL <= '1';

			FIFO_AEMPTY <= '1';
			FIFO_EMPTY <= '1';

			Counter_XXX <= (others => '0');

		elsif rising_edge(Clk) then

			Counter_XXX <= Counter_XXX; 

		    if FIFO_OUT_VALID = '1' then
--			if FIFO_IN_READENABLE_DX = '1' then
				Counter_XXX <= Counter_XXX + 1;
			else
				null;
			end if;

			din_DECOUPLE <= FIFO_OUT_DATA;
--			din_DECOUPLE(7 downto 1) <= std_logic_vector(Counter_XXX);
			wr_en_DECOUPLE <= FIFO_OUT_VALID;

			dout <= dout_DECOUPLE;
			-- dout(7 downto 1) <= std_logic_vector(Counter_XXX);
			rd_en_DECOUPLE <= FIFO_IN_READENABLE_DX;

			FIFO_AFULL <= prog_full_DECOUPLE;
			FIFO_FULL <= full_DECOUPLE;

			FIFO_AEMPTY <= prog_empty_DECOUPLE;
			FIFO_EMPTY <= empty_DECOUPLE;
		end if;
	end process DECOUPLE_FIFO;		

	-- We need to reverse the bit order otherwise the results is in the wrong order...
	FIFO_IN_DATA(7) <= dout(0);
	FIFO_IN_DATA(6) <= dout(1);
	FIFO_IN_DATA(5) <= dout(2);
	FIFO_IN_DATA(4) <= dout(3);
	FIFO_IN_DATA(3) <= dout(4);
	FIFO_IN_DATA(2) <= dout(5);
	FIFO_IN_DATA(1) <= dout(6);
	FIFO_IN_DATA(0) <= dout(7);
	
	-- If there is still space then '1'
	ExportNext128ChannelBlock <= NOT FIFO_AFULL;
	-- If there is are at least 14 x 8 bit left, be a '1'
	FIFO_IN_VALID <= NOT FIFO_AEMPTY;

	-- We need to cross the read signal from the 
	-- slow Zarlink module to the fast fifo.
	WR_EN_FIFO : process (Clk,Reset)
	begin
		if Reset = '1' then
			FIFO_IN_READENABLE_DX <= '0';
		elsif rising_edge(Clk) then
			FIFO_IN_READENABLE_DX <= '0';
			FIFO_IN_READENABLE_LAST <= FIFO_IN_READENABLE;

		
			if (FIFO_IN_READENABLE_LAST = '0') AND (FIFO_IN_READENABLE = '1') then
				FIFO_IN_READENABLE_DX <= '1';
			else
				null;
			end if;
		
		end if;
	end process WR_EN_FIFO;		

	ZL_DEBUG : process (SPI_CLK_BUFFER,SPI_ZL_CS_BUFFER)
	begin
		if SPI_ZL_CS_BUFFER = '1' then
			DEBUG_ZARLINK_DATA_INTERNAL <= (others => '0');
			DEBUG_ZARLINK_VALID_INTERNAL <= '0';
			DEBUG_COUNTER <= (others => '0');
		elsif rising_edge(SPI_CLK_BUFFER) then
			
			if DEBUG_COUNTER = to_unsigned(7,4) then
				DEBUG_ZARLINK_VALID_INTERNAL <= '1';
				DEBUG_COUNTER <= (others => '0');
				DEBUG_ZARLINK_DATA_INTERNAL(7 downto 0) <= DEBUG_ZARLINK_DATA_INTERNAL(6 downto 0) & SPI_SDO;
				DEBUG_ZARLINK_DATA_INTERNAL(15 downto 8) <= DEBUG_ZARLINK_DATA_INTERNAL(14 downto 8) & SPI_SDI_BUFFER;
			else
				DEBUG_ZARLINK_VALID_INTERNAL <= '0';
				DEBUG_COUNTER <= DEBUG_COUNTER + 1;
				DEBUG_ZARLINK_DATA_INTERNAL(7 downto 0) <= DEBUG_ZARLINK_DATA_INTERNAL(6 downto 0) & SPI_SDO;
				DEBUG_ZARLINK_DATA_INTERNAL(15 downto 8) <= DEBUG_ZARLINK_DATA_INTERNAL(14 downto 8) & SPI_SDI_BUFFER;
			end if;
		
		end if;
	end process ZL_DEBUG;		

	-- First we let the FIFO finish its reset then we let the 
	-- rest come to life... 
	DELAY : process (Clk,Reset)
	begin
		if Reset = '1' then
			ResetRHAandFIFO_DELAYED <= '1';
			RHA_RESET_COUNTER <= (others => '0');
		elsif rising_edge(Clk) then	
			RHA_RESET_COUNTER <= RHA_RESET_COUNTER;
			if ResetRHAandFIFO = '1' then
				RHA_RESET_COUNTER <= (others => '0');
				ResetRHAandFIFO_DELAYED <= '1';
			elsif RHA_RESET_COUNTER = to_unsigned(10,5) then 
				ResetRHAandFIFO_DELAYED <= '0';
			else
				RHA_RESET_COUNTER <= RHA_RESET_COUNTER + 1;
				ResetRHAandFIFO_DELAYED <= '1';
			end if;
		end if;
	end process DELAY;			
	
	
	RHA_Reset <= NOT (ResetRHAandFIFO_DELAYED & ResetRHAandFIFO_DELAYED & ResetRHAandFIFO_DELAYED & ResetRHAandFIFO_DELAYED &
					 ResetRHAandFIFO_DELAYED & ResetRHAandFIFO_DELAYED & ResetRHAandFIFO_DELAYED & ResetRHAandFIFO_DELAYED);
	
	Inst_CONTROL_NEXUS: entity work.CONTROL_NEXUS 
	PORT MAP(
		SPI_ZL_CS => SPI_ZL_CS_BUFFER,
		SPI_SDI => SPI_SDI_BUFFER,
		SPI_SDO => SPI_SDO,
		SPI_CLK => SPI_CLK_BUFFER,
		ResetRHAandFIFO => ResetRHAandFIFO,
		ChannelMask => ChannelMask,
		ADCResolution => ADCResolution,
		SampleRate => SampleRate,
		Filter => Filter,
		DeviceID => DeviceID,
		BITMASK_SAMPLE => BITMASK_SAMPLE,
		Clk_Reduced => Clk_Reduced,
		FIFO_IN_DATA => FIFO_IN_DATA,
		FIFO_IN_VALID => FIFO_IN_VALID,
		FIFO_IN_READENABLE => FIFO_IN_READENABLE,
		TestMode => TestMode,
		DEBUG_STRING_ORGANIZER => DEBUG_STRING_ORGANIZER,
		Reset => Reset,
		Clk_FAST => Clk_FAST,
		Clk => Clk
	);

	Inst_DATA_ACQUISITION_BLOCK: entity work.DATA_ACQUISITION_BLOCK 
	PORT MAP(
		RHA_ADC_out => RHA_ADC_out,
		RHA_sync => RHA_sync,
		RHA_ADC_clk => RHA_ADC_clk,
		FIFO_OUT_DATA => FIFO_OUT_DATA,
		FIFO_OUT_VALID => FIFO_OUT_VALID,
		ExportNext128ChannelBlock => ExportNext128ChannelBlock,
		ChannelMask => ChannelMask,
		ADCResolution => ADCResolution,
		SampleRate => SampleRate,
		Filter => Filter,
		DeviceID => DeviceID,
		TestMode => TestMode,
		BITMASK_SAMPLE => BITMASK_SAMPLE,
		ERROR_RHA_ASYNC_FIRST_OUT => ERROR_RHA_ASYNC_FIRST_OUT,
		ERROR_RHA_ASYNC_VALID_OUT => ERROR_RHA_ASYNC_VALID_OUT,
		ERROR_PARITY_OUT => ERROR_PARITY_OUT,
		ERROR_PATTERN_OUT => ERROR_PATTERN_OUT,
		ERROR_ADDRESS_OUT => ERROR_ADDRESS_OUT,
		ERROR_BUFFEROVERFLOW_OUT => ERROR_BUFFEROVERFLOW_OUT,
		ERROR_ALL_OUT => ERROR_ALL_OUT,
		Clk_Slow => Clk_Reduced,
		Reset => ResetRHAandFIFO_DELAYED,
		Clk => Clk
	);

	CLK_2 : process (Clk_FAST,Reset)
	begin
		if Reset = '1' then
			Clk <= '0';
		elsif rising_edge(Clk_FAST) then
			Clk <= NOT Clk;
		end if;
	end process CLK_2;


end architecture Behavioral;

