----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    18:04:15 07/11/2014 
-- Design Name: 
-- Module Name:    RHA_ARRAY - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

-- This module operates the RHA statemachines and collects these data. 
-- Then the data is cut into single words and send out. 
-- This module notifies if the sync pulses of the selected (channel mask)
-- RHAs get strange. 

entity RHA_ARRAY is
    Port ( 
		-- For the data stream
		RHA_ADC_out 				: in  STD_LOGIC_VECTOR(7 downto 0);
		-- Where a RHA data frame starts 
		RHA_sync 					: in  STD_LOGIC_VECTOR(7 downto 0);
		-- RHA ADC clock
		RHA_ADC_clk					: out STD_LOGIC_VECTOR(7 downto 0);

		-- ACTIVE channel
		RHA_CHANNEL_MASK			: in STD_LOGIC_VECTOR(127 downto 0);
		
        -- The collected output, brocken into single values       
	 	DATAOUT_DATA_OUT_EXT		: out STD_LOGIC_VECTOR(15 downto 0);
		-- VALID is checking if this is from an active RHA channel (according to the
		-- channel mask). 
		DATAOUT_VALID_OUT_EXT		: out STD_LOGIC;
		-- Channel_Number_OUT		: out STD_LOGIC_VECTOR(3 downto 0); 
		NewBlock						: out STD_LOGIC; 
		
		ReadRequest					: in  STD_LOGIC;

      -- Errors (tells us if the RHA get asynchronous in their sync pulses) 
		ERROR_RHA_ASYNC_FIRST   : out STD_LOGIC;
		ERROR_RHA_ASYNC_VALID   : out STD_LOGIC;		
		-- Is there a parity problem in this sample?
		ERROR_PARITY 				: out STD_LOGIC;
		-- Is there a pattern problem in this sample?
		ERROR_PATTERN 				: out STD_LOGIC;
		-- Is there an address problem in this sample?
		ERROR_ADDRESS 				: out STD_LOGIC;
		-- Was the readout too slow?
		ERROR_BUFFEROVERFLOW		: out STD_LOGIC;

		
		-- The clock for collecting data from the RHAs (10KHz all channels = 8MHz)
		-- and for data processing 
		Clk 			        		: in  STD_LOGIC;
		Clk_Slow		        		: in  STD_LOGIC;
        -- Reset to the RHA state machines
		Reset_RHA	            : in  STD_LOGIC;	 
        -- Reset for this module 
      Reset 		            : in  STD_LOGIC	 
	 
	 );
end RHA_ARRAY;

architecture Behavioral of RHA_ARRAY is

		-- 0 
		signal DATAOUT_DATA_0		: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DATAOUT_VALID_0		: STD_LOGIC := '0';
		signal DATAOUT_FIRST_0		: STD_LOGIC := '0';
		signal DATAOUT_ADDRESS_0	: STD_LOGIC_VECTOR(4 downto 0) := (others => '0');
		signal DATAOUT_PATTERN_0	: STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
		signal DATAOUT_PARITY_0		: STD_LOGIC := '0';

		-- 1
		signal DATAOUT_DATA_1		: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DATAOUT_VALID_1		: STD_LOGIC := '0';
		signal DATAOUT_FIRST_1		: STD_LOGIC := '0';
		signal DATAOUT_ADDRESS_1	: STD_LOGIC_VECTOR(4 downto 0) := (others => '0');
		signal DATAOUT_PATTERN_1	: STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
		signal DATAOUT_PARITY_1		: STD_LOGIC := '0';

		-- 2
		signal DATAOUT_DATA_2		: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DATAOUT_VALID_2		: STD_LOGIC := '0';
		signal DATAOUT_FIRST_2		: STD_LOGIC := '0';
		signal DATAOUT_ADDRESS_2	: STD_LOGIC_VECTOR(4 downto 0) := (others => '0');
		signal DATAOUT_PATTERN_2	: STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
		signal DATAOUT_PARITY_2		: STD_LOGIC := '0';

		-- 3
		signal DATAOUT_DATA_3		: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DATAOUT_VALID_3		: STD_LOGIC := '0';
		signal DATAOUT_FIRST_3		: STD_LOGIC := '0';
		signal DATAOUT_ADDRESS_3	: STD_LOGIC_VECTOR(4 downto 0) := (others => '0');
		signal DATAOUT_PATTERN_3	: STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
		signal DATAOUT_PARITY_3		: STD_LOGIC := '0';

		-- 4 
		signal DATAOUT_DATA_4		: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DATAOUT_VALID_4		: STD_LOGIC := '0';
		signal DATAOUT_FIRST_4		: STD_LOGIC := '0';
		signal DATAOUT_ADDRESS_4	: STD_LOGIC_VECTOR(4 downto 0) := (others => '0');
		signal DATAOUT_PATTERN_4	: STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
		signal DATAOUT_PARITY_4		: STD_LOGIC := '0';		

		-- 5
		signal DATAOUT_DATA_5		: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DATAOUT_VALID_5		: STD_LOGIC := '0';
		signal DATAOUT_FIRST_5		: STD_LOGIC := '0';
		signal DATAOUT_ADDRESS_5	: STD_LOGIC_VECTOR(4 downto 0) := (others => '0');
		signal DATAOUT_PATTERN_5	: STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
		signal DATAOUT_PARITY_5		: STD_LOGIC := '0';

		-- 6
		signal DATAOUT_DATA_6		: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DATAOUT_VALID_6		: STD_LOGIC := '0';
		signal DATAOUT_FIRST_6		: STD_LOGIC := '0';
		signal DATAOUT_ADDRESS_6	: STD_LOGIC_VECTOR(4 downto 0) := (others => '0');
		signal DATAOUT_PATTERN_6	: STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
		signal DATAOUT_PARITY_6		: STD_LOGIC := '0';

		-- 7
		signal DATAOUT_DATA_7		: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DATAOUT_VALID_7		: STD_LOGIC := '0';
		signal DATAOUT_FIRST_7		: STD_LOGIC := '0';
		signal DATAOUT_ADDRESS_7	: STD_LOGIC_VECTOR(4 downto 0) := (others => '0');
		signal DATAOUT_PATTERN_7	: STD_LOGIC_VECTOR(2 downto 0) := (others => '0');
		signal DATAOUT_PARITY_7		: STD_LOGIC := '0';

		-- For ignoring the missing RHAs by the user (RHA_SELECTION_MASK)
		signal GATED_VALID			: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal GATED_FIRST			: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');

		-- Delayed copies 
		signal GATED_VALID_LAST		: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal GATED_FIRST_LAST		: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		

		-- We want to know which channel the data is from (for throwing the data away)
		signal Channel_Number		: unsigned(3 downto 0) := (others=>'0'); 
				
		-- Which RHA should be checked (defined by the channel masl)
		signal RHA_SELECTION_MASK	: STD_LOGIC_VECTOR(7 downto 0) := (others=>'0');

		-- Buffering of the data 
		signal DATAOUT_DATA_INTERNAL		: STD_LOGIC_VECTOR(127 downto 0) := (others => '0');
      	signal DATAOUT_VALIDSTRING      	: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal DATAOUT_ADDRESS_INTERNAL	: STD_LOGIC_VECTOR(39 downto 0) := (others => '0');
		signal DATAOUT_PATTERN_INTERNAL	: STD_LOGIC_VECTOR(23 downto 0) := (others => '0');
		signal DATAOUT_PARITY_INTERNAL	: STD_LOGIC_VECTOR(7 downto 0) := (others => '0');

		signal ERROR_PARITY_INTERNAL		: STD_LOGIC := '0';
		signal ERROR_PATTERN_INTERNAL		: STD_LOGIC := '0';
		signal ERROR_ADDRESS_INTERNAL		: STD_LOGIC := '0';
		signal ERROR_RHA_ASYNC_FIRST_INTERNAL 	: STD_LOGIC := '0';
		signal ERROR_RHA_ASYNC_VALID_INTERNAL 	: STD_LOGIC := '0';
		signal BufferOverflow_INTERNAL 	: STD_LOGIC := '0';
		
		signal ChannelMask_Sub_0			: STD_LOGIC_VECTOR(14 downto 0) := (others => '0');	
		signal ChannelMask_Sub_1			: STD_LOGIC_VECTOR(14 downto 0) := (others => '0');	
		signal ChannelMask_Sub_2			: STD_LOGIC_VECTOR(14 downto 0) := (others => '0');	
		signal ChannelMask_Sub_3			: STD_LOGIC_VECTOR(14 downto 0) := (others => '0');	
		signal ChannelMask_Sub_4			: STD_LOGIC_VECTOR(14 downto 0) := (others => '0');	
		signal ChannelMask_Sub_5			: STD_LOGIC_VECTOR(14 downto 0) := (others => '0');	
		signal ChannelMask_Sub_6			: STD_LOGIC_VECTOR(14 downto 0) := (others => '0');	
		signal ChannelMask_Sub_7			: STD_LOGIC_VECTOR(14 downto 0) := (others => '0');	

		constant AddressCheckInfo_Constant : STD_LOGIC_VECTOR(79 downto 0) := 
				"01000" & "01001" & "01010" & "01011" &
				"01100" & "01101" & "01110" & "01111" &
				"10000" & "10001" & "10010" & "10011" &
				"10100" & "10101" & "10110" & "10111";
		signal AddressCheckInfo					: STD_LOGIC_VECTOR(79 downto 0) := (others => '0');


	 	signal DATAOUT_DATA_OUT		: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal DATAOUT_VALID_OUT	: STD_LOGIC := '0';	

--		signal Test_Counter 		: unsigned (10 downto 0) := (others => '0');

begin
	ERROR_BUFFEROVERFLOW <= BufferOverflow_INTERNAL;
	
	-- 0
	RHA0 : entity work.RHA_DATA_COLLECTOR 
    Port MAP ( 
			RHA_ADC_out => RHA_ADC_out(0),
			RHA_sync => RHA_sync(0),
			RHA_ADC_clk	=> RHA_ADC_clk(0),
			DATAOUT_DATA => DATAOUT_DATA_0,
			DATAOUT_VALID => DATAOUT_VALID_0,
			DATAOUT_FIRST => DATAOUT_FIRST_0,
			DATAOUT_ADDRESS => DATAOUT_ADDRESS_0,
			DATAOUT_PATTERN => DATAOUT_PATTERN_0,
			DATAOUT_PARITY => DATAOUT_PARITY_0,			
			Clk => Clk_Slow,
			Reset => Reset_RHA
		);

	-- 1
	RHA1 : entity work.RHA_DATA_COLLECTOR 
    Port MAP ( 
			RHA_ADC_out => RHA_ADC_out(1),
			RHA_sync => RHA_sync(1),
			RHA_ADC_clk	=> RHA_ADC_clk(1),
			DATAOUT_DATA => DATAOUT_DATA_1,
			DATAOUT_VALID => DATAOUT_VALID_1,
			DATAOUT_FIRST => DATAOUT_FIRST_1,
			DATAOUT_ADDRESS => DATAOUT_ADDRESS_1,
			DATAOUT_PATTERN => DATAOUT_PATTERN_1,
			DATAOUT_PARITY => DATAOUT_PARITY_1,			
			Clk => Clk_Slow,
			Reset => Reset_RHA
		);

	-- 2
	RHA2 : entity work.RHA_DATA_COLLECTOR 
    Port MAP ( 
			RHA_ADC_out => RHA_ADC_out(2),
			RHA_sync => RHA_sync(2),
			RHA_ADC_clk	=> RHA_ADC_clk(2),
			DATAOUT_DATA => DATAOUT_DATA_2,
			DATAOUT_VALID => DATAOUT_VALID_2,
			DATAOUT_FIRST => DATAOUT_FIRST_2,
			DATAOUT_ADDRESS => DATAOUT_ADDRESS_2,
			DATAOUT_PATTERN => DATAOUT_PATTERN_2,
			DATAOUT_PARITY => DATAOUT_PARITY_2,			
			Clk => Clk_Slow,
			Reset => Reset_RHA
		);

	-- 3
	RHA3 : entity work.RHA_DATA_COLLECTOR 
    Port MAP ( 
			RHA_ADC_out => RHA_ADC_out(3),
			RHA_sync => RHA_sync(3),
			RHA_ADC_clk	=> RHA_ADC_clk(3),
			DATAOUT_DATA => DATAOUT_DATA_3,
			DATAOUT_VALID => DATAOUT_VALID_3,
			DATAOUT_FIRST => DATAOUT_FIRST_3,
			DATAOUT_ADDRESS => DATAOUT_ADDRESS_3,
			DATAOUT_PATTERN => DATAOUT_PATTERN_3,
			DATAOUT_PARITY => DATAOUT_PARITY_3,			
			Clk => Clk_Slow,
			Reset => Reset_RHA
		);

	-- 4
	RHA4 : entity work.RHA_DATA_COLLECTOR 
    Port MAP ( 
			RHA_ADC_out => RHA_ADC_out(4),
			RHA_sync => RHA_sync(4),
			RHA_ADC_clk	=> RHA_ADC_clk(4),
			DATAOUT_DATA => DATAOUT_DATA_4,
			DATAOUT_VALID => DATAOUT_VALID_4,
			DATAOUT_FIRST => DATAOUT_FIRST_4,
			DATAOUT_ADDRESS => DATAOUT_ADDRESS_4,
			DATAOUT_PATTERN => DATAOUT_PATTERN_4,
			DATAOUT_PARITY => DATAOUT_PARITY_4,			
			Clk => Clk_Slow,
			Reset => Reset_RHA
		);

	-- 5
	RHA5 : entity work.RHA_DATA_COLLECTOR 
    Port MAP ( 
			RHA_ADC_out => RHA_ADC_out(5),
			RHA_sync => RHA_sync(5),
			RHA_ADC_clk	=> RHA_ADC_clk(5),
			DATAOUT_DATA => DATAOUT_DATA_5,
			DATAOUT_VALID => DATAOUT_VALID_5,
			DATAOUT_FIRST => DATAOUT_FIRST_5,
			DATAOUT_ADDRESS => DATAOUT_ADDRESS_5,
			DATAOUT_PATTERN => DATAOUT_PATTERN_5,
			DATAOUT_PARITY => DATAOUT_PARITY_5,			
			Clk => Clk_Slow,
			Reset => Reset_RHA
		);

	-- 6
	RHA6 : entity work.RHA_DATA_COLLECTOR 
    Port MAP ( 
			RHA_ADC_out => RHA_ADC_out(6),
			RHA_sync => RHA_sync(6),
			RHA_ADC_clk	=> RHA_ADC_clk(6),
			DATAOUT_DATA => DATAOUT_DATA_6,
			DATAOUT_VALID => DATAOUT_VALID_6,
			DATAOUT_FIRST => DATAOUT_FIRST_6,
			DATAOUT_ADDRESS => DATAOUT_ADDRESS_6,
			DATAOUT_PATTERN => DATAOUT_PATTERN_6,
			DATAOUT_PARITY => DATAOUT_PARITY_6,			
			Clk => Clk_Slow,
			Reset => Reset_RHA
		);

	-- 7
	RHA7 : entity work.RHA_DATA_COLLECTOR 
    Port MAP ( 
			RHA_ADC_out => RHA_ADC_out(7),
			RHA_sync => RHA_sync(7),
			RHA_ADC_clk	=> RHA_ADC_clk(7),
			DATAOUT_DATA => DATAOUT_DATA_7,
			DATAOUT_VALID => DATAOUT_VALID_7,
			DATAOUT_FIRST => DATAOUT_FIRST_7,
			DATAOUT_ADDRESS => DATAOUT_ADDRESS_7,
			DATAOUT_PATTERN => DATAOUT_PATTERN_7,
			DATAOUT_PARITY => DATAOUT_PARITY_7,			
			Clk =>Clk_Slow,
			Reset => Reset_RHA
		);

	-- We care only about RHAs which we expect to work (VALID SIGNAL)
	GATED_VALID(0) <= DATAOUT_VALID_0 AND RHA_SELECTION_MASK(0);
	GATED_VALID(1) <= DATAOUT_VALID_1 AND RHA_SELECTION_MASK(1);
	GATED_VALID(2) <= DATAOUT_VALID_2 AND RHA_SELECTION_MASK(2);
	GATED_VALID(3) <= DATAOUT_VALID_3 AND RHA_SELECTION_MASK(3);
	GATED_VALID(4) <= DATAOUT_VALID_4 AND RHA_SELECTION_MASK(4);
	GATED_VALID(5) <= DATAOUT_VALID_5 AND RHA_SELECTION_MASK(5);
	GATED_VALID(6) <= DATAOUT_VALID_6 AND RHA_SELECTION_MASK(6);
	GATED_VALID(7) <= DATAOUT_VALID_7 AND RHA_SELECTION_MASK(7);

	-- We care only about RHAs which we expect to work (FIRST SIGNAL)
	GATED_FIRST(0) <= DATAOUT_FIRST_0 AND RHA_SELECTION_MASK(0);
	GATED_FIRST(1) <= DATAOUT_FIRST_1 AND RHA_SELECTION_MASK(1);
	GATED_FIRST(2) <= DATAOUT_FIRST_2 AND RHA_SELECTION_MASK(2);
	GATED_FIRST(3) <= DATAOUT_FIRST_3 AND RHA_SELECTION_MASK(3);
	GATED_FIRST(4) <= DATAOUT_FIRST_4 AND RHA_SELECTION_MASK(4);
	GATED_FIRST(5) <= DATAOUT_FIRST_5 AND RHA_SELECTION_MASK(5);
	GATED_FIRST(6) <= DATAOUT_FIRST_6 AND RHA_SELECTION_MASK(6);
	GATED_FIRST(7) <= DATAOUT_FIRST_7 AND RHA_SELECTION_MASK(7);
	
	-- We ask the channel mask which RHA has to be available 
	RHA_SELECTION_MASK_PROCESS : process (RHA_CHANNEL_MASK)
	begin
		-- 0
		if RHA_CHANNEL_MASK(15 downto 0) = "0000000000000000" then
			RHA_SELECTION_MASK(0) <= '0';
		else
			RHA_SELECTION_MASK(0) <= '1';
		end if;

		-- 1
		if RHA_CHANNEL_MASK(31 downto 16) = "0000000000000000" then
			RHA_SELECTION_MASK(1) <= '0';
		else
			RHA_SELECTION_MASK(1) <= '1';
		end if;

		-- 2
		if RHA_CHANNEL_MASK(47 downto 32) = "0000000000000000" then
			RHA_SELECTION_MASK(2) <= '0';
		else
			RHA_SELECTION_MASK(2) <= '1';
		end if;

		-- 3
		if RHA_CHANNEL_MASK(63 downto 48) = "0000000000000000" then
			RHA_SELECTION_MASK(3) <= '0';
		else
			RHA_SELECTION_MASK(3) <= '1';
		end if;

		-- 4
		if RHA_CHANNEL_MASK(79 downto 64) = "0000000000000000" then
			RHA_SELECTION_MASK(4) <= '0';
		else
			RHA_SELECTION_MASK(4) <= '1';
		end if;

		-- 5
		if RHA_CHANNEL_MASK(95 downto 80) = "0000000000000000" then
			RHA_SELECTION_MASK(5) <= '0';
		else
			RHA_SELECTION_MASK(5) <= '1';
		end if;

		-- 6
		if RHA_CHANNEL_MASK(111 downto 96) = "0000000000000000" then
			RHA_SELECTION_MASK(6) <= '0';
		else
			RHA_SELECTION_MASK(6) <= '1';
		end if;

		-- 7
		if RHA_CHANNEL_MASK(127 downto 112) = "0000000000000000" then
			RHA_SELECTION_MASK(7) <= '0';
		else
			RHA_SELECTION_MASK(7) <= '1';
		end if;
	end process RHA_SELECTION_MASK_PROCESS;

	-- Checking the acutal RHA sample for errors 
	DataChecker: entity work. RHA_DATA_CHECKER 
    Port MAP (
		DATAOUT_DATA => DATAOUT_DATA_INTERNAL(15 downto 0),
		DATAOUT_ADDRESS => DATAOUT_ADDRESS_INTERNAL(4 downto 0),
		DATAOUT_PATTERN => DATAOUT_PATTERN_INTERNAL(2 downto 0),
		DATAOUT_PARITY	=> DATAOUT_PARITY_INTERNAL(0),
		DATAOUT_ADDRESS_COMP	=> AddressCheckInfo(79 downto 75),
		ERROR_PARITY => ERROR_PARITY_INTERNAL,
		ERROR_PATTERN => ERROR_PATTERN_INTERNAL,
		ERROR_ADDRESS => ERROR_ADDRESS_INTERNAL
	 );

	-- Make a copy of the data before it gets destroyed 
	COPY_VALID_DATA : process (Clk,Reset)
	begin
		if Reset = '1' then
			DATAOUT_DATA_INTERNAL <= (others => '0');
			DATAOUT_ADDRESS_INTERNAL <= (others => '0');
			DATAOUT_PATTERN_INTERNAL <= (others => '0');
			DATAOUT_PARITY_INTERNAL <= (others => '0');
         	DATAOUT_VALIDSTRING <= (others => '0');
	      	DATAOUT_DATA_OUT <= (others => '0');
		   	DATAOUT_VALID_OUT <= '0';
	 		DATAOUT_DATA_OUT_EXT <= (others => '0');
			DATAOUT_VALID_OUT_EXT <= '0';	
			
			ChannelMask_Sub_0 <= (others => '0');
			ChannelMask_Sub_1 <= (others => '0');
			ChannelMask_Sub_2 <= (others => '0');
			ChannelMask_Sub_3 <= (others => '0');
			ChannelMask_Sub_4 <= (others => '0');
			ChannelMask_Sub_5 <= (others => '0');
			ChannelMask_Sub_6 <= (others => '0');
			ChannelMask_Sub_7 <= (others => '0');

			GATED_VALID_LAST <= (others => '0');
			GATED_FIRST_LAST <= (others => '0');
			NewBlock <= '0';
			
			AddressCheckInfo <= (others => '0');

			ERROR_PARITY <= '0';
			ERROR_PATTERN <= '0';
			ERROR_ADDRESS <= '0';
			BufferOverflow_INTERNAL <= '0';
			
--			Test_Counter <= (others => '0');
		elsif rising_edge(Clk) then

         -- Default 
			DATAOUT_DATA_INTERNAL <= DATAOUT_DATA_INTERNAL;
			DATAOUT_ADDRESS_INTERNAL <= DATAOUT_ADDRESS_INTERNAL;
			DATAOUT_PATTERN_INTERNAL <= DATAOUT_PATTERN_INTERNAL;
			DATAOUT_PARITY_INTERNAL <= DATAOUT_PARITY_INTERNAL;			
         	DATAOUT_VALIDSTRING <= DATAOUT_VALIDSTRING;

			ERROR_PARITY <= '0';
			ERROR_PATTERN <= '0';
			ERROR_ADDRESS <= '0';
			BufferOverflow_INTERNAL <= BufferOverflow_INTERNAL;
			
			AddressCheckInfo <=  AddressCheckInfo;
			
			ChannelMask_Sub_0 <= ChannelMask_Sub_0;
			ChannelMask_Sub_1 <= ChannelMask_Sub_1;
			ChannelMask_Sub_2 <= ChannelMask_Sub_2;
			ChannelMask_Sub_3 <= ChannelMask_Sub_3;
			ChannelMask_Sub_4 <= ChannelMask_Sub_4;
			ChannelMask_Sub_5 <= ChannelMask_Sub_5;
			ChannelMask_Sub_6 <= ChannelMask_Sub_6;
			ChannelMask_Sub_7 <= ChannelMask_Sub_7;

	      	DATAOUT_DATA_OUT <= (others => '0');
		   	DATAOUT_VALID_OUT <= '0';

	 		DATAOUT_DATA_OUT_EXT <= DATAOUT_DATA_OUT;
			DATAOUT_VALID_OUT_EXT <= DATAOUT_VALID_OUT;	

			GATED_VALID_LAST <= GATED_VALID;
			GATED_FIRST_LAST <= GATED_FIRST;
--			Test_Counter <= Test_Counter;

			if (NOT(GATED_VALID = "00000000")) AND (GATED_VALID_LAST = "00000000") then
				DATAOUT_DATA_INTERNAL <= DATAOUT_DATA_7 & DATAOUT_DATA_6 & DATAOUT_DATA_5 & DATAOUT_DATA_4 &
								DATAOUT_DATA_3 & DATAOUT_DATA_2 & DATAOUT_DATA_1 & DATAOUT_DATA_0;

--				DATAOUT_DATA_INTERNAL <= 
--				"11" & std_logic_vector(Test_Counter) & "11" & DATAOUT_DATA_7(0) &
--				"11" & std_logic_vector(Test_Counter) & "11" & DATAOUT_DATA_6(0) &
--				"11" & std_logic_vector(Test_Counter) & "11" & DATAOUT_DATA_5(0) &
--				"11" & std_logic_vector(Test_Counter) & "11" & DATAOUT_DATA_4(0) &
--				"11" & std_logic_vector(Test_Counter) & "11" & DATAOUT_DATA_3(0) &
--				"11" & std_logic_vector(Test_Counter) & "11" & DATAOUT_DATA_2(0) &
--				"11" & std_logic_vector(Test_Counter) & "11" & DATAOUT_DATA_1(0) &
--				"11" & std_logic_vector(Test_Counter) & "11" & DATAOUT_DATA_0(0) ;



				DATAOUT_ADDRESS_INTERNAL <= DATAOUT_ADDRESS_7 & DATAOUT_ADDRESS_6 & DATAOUT_ADDRESS_5 & DATAOUT_ADDRESS_4 &
								DATAOUT_ADDRESS_3 & DATAOUT_ADDRESS_2 & DATAOUT_ADDRESS_1 & DATAOUT_ADDRESS_0;
								
				DATAOUT_PATTERN_INTERNAL <= DATAOUT_PATTERN_7 & DATAOUT_PATTERN_6 & DATAOUT_PATTERN_5 & DATAOUT_PATTERN_4 &
								DATAOUT_PATTERN_3 & DATAOUT_PATTERN_2 & DATAOUT_PATTERN_1 & DATAOUT_PATTERN_0;
								
				DATAOUT_PARITY_INTERNAL <= DATAOUT_PARITY_7 & DATAOUT_PARITY_6 & DATAOUT_PARITY_5 & DATAOUT_PARITY_4 &
								DATAOUT_PARITY_3 & DATAOUT_PARITY_2 & DATAOUT_PARITY_1 & DATAOUT_PARITY_0;
				
				-- Here we will handle the user's channel mask 
            	if (NOT(GATED_FIRST = "00000000")) AND (GATED_FIRST_LAST = "00000000") then
					ChannelMask_Sub_0 <= RHA_CHANNEL_MASK(15 downto 1);
					ChannelMask_Sub_1 <= RHA_CHANNEL_MASK(31 downto 17);
					ChannelMask_Sub_2 <= RHA_CHANNEL_MASK(47 downto 33);
					ChannelMask_Sub_3 <= RHA_CHANNEL_MASK(63 downto 49);
					ChannelMask_Sub_4 <= RHA_CHANNEL_MASK(79 downto 65);
					ChannelMask_Sub_5 <= RHA_CHANNEL_MASK(95 downto 81);
					ChannelMask_Sub_6 <= RHA_CHANNEL_MASK(111 downto 97);
					ChannelMask_Sub_7 <= RHA_CHANNEL_MASK(127 downto 113);
					DATAOUT_VALIDSTRING <= RHA_CHANNEL_MASK(112) & RHA_CHANNEL_MASK(96) & RHA_CHANNEL_MASK(80) & RHA_CHANNEL_MASK(64) &
													RHA_CHANNEL_MASK(48) & RHA_CHANNEL_MASK(32) & RHA_CHANNEL_MASK(16) & RHA_CHANNEL_MASK(0);
--					DATAOUT_VALIDSTRING <= (others => '1');
					AddressCheckInfo <= AddressCheckInfo_Constant;
					NewBlock <= '1';
				else
					ChannelMask_Sub_0 <= "0" & ChannelMask_Sub_0(14 downto 1);
					ChannelMask_Sub_1 <= "0" & ChannelMask_Sub_1(14 downto 1);
					ChannelMask_Sub_2 <= "0" & ChannelMask_Sub_2(14 downto 1);
					ChannelMask_Sub_3 <= "0" & ChannelMask_Sub_3(14 downto 1);
					ChannelMask_Sub_4 <= "0" & ChannelMask_Sub_4(14 downto 1);
					ChannelMask_Sub_5 <= "0" & ChannelMask_Sub_5(14 downto 1);
					ChannelMask_Sub_6 <= "0" & ChannelMask_Sub_6(14 downto 1);
					ChannelMask_Sub_7 <= "0" & ChannelMask_Sub_7(14 downto 1);
					DATAOUT_VALIDSTRING <= ChannelMask_Sub_7(0) & ChannelMask_Sub_6(0) & ChannelMask_Sub_5(0) & ChannelMask_Sub_4(0) & 
													ChannelMask_Sub_3(0) & ChannelMask_Sub_2(0) & ChannelMask_Sub_1(0) & ChannelMask_Sub_0(0);
--					DATAOUT_VALIDSTRING <= (others => '1');
					AddressCheckInfo <= AddressCheckInfo(74 downto 0) & "00000";
					NewBlock <= '0';
				end if;
				
				-- Check for Buffer-Overruns 
				if DATAOUT_VALIDSTRING = "00000000" then	 
					BufferOverflow_INTERNAL <= '0';
				else
					BufferOverflow_INTERNAL <= '1';
				end if;
				
		    elsif (ReadRequest = '1') OR (DATAOUT_VALIDSTRING(0) = '0')  then
            -- Bit shift the used data continously away
            -- The VALIDSTRING takes care of invalid data  
				DATAOUT_DATA_INTERNAL <= "0000000000000000" & DATAOUT_DATA_INTERNAL(127 downto 16);
				DATAOUT_ADDRESS_INTERNAL <= "00000" & DATAOUT_ADDRESS_INTERNAL(39 downto 5);
				DATAOUT_PATTERN_INTERNAL <= "000" & DATAOUT_PATTERN_INTERNAL(23 downto 3);
				DATAOUT_PARITY_INTERNAL <= "0" & DATAOUT_PARITY_INTERNAL(7 downto 1);
            	DATAOUT_VALIDSTRING <= "0" & DATAOUT_VALIDSTRING(7 downto 1);
              
            -- This is were the output is continously generated 

				ERROR_PARITY <= ERROR_PARITY_INTERNAL;
				ERROR_PATTERN <= ERROR_PATTERN_INTERNAL;
				ERROR_ADDRESS <= ERROR_ADDRESS_INTERNAL;
				
				NewBlock <= '0';
--				if DATAOUT_VALIDSTRING(0) = '1' then
--					Test_Counter <= Test_Counter + 1;
--				else
--					null;
--				end if;

			else
            -- This is were the output is continously generated 
	          	DATAOUT_DATA_OUT <= DATAOUT_DATA_INTERNAL(15 downto 0);
			  	--DATAOUT_DATA_OUT (15 downto 1) <= "11" & std_logic_vector(Test_Counter) & "11";
		      	DATAOUT_VALID_OUT <= DATAOUT_VALIDSTRING(0);

				null;
			end if;
		
		end if;
	end process COPY_VALID_DATA;

	ERROR_RHA_ASYNC_FIRST <= ERROR_RHA_ASYNC_FIRST_INTERNAL;
	ERROR_RHA_ASYNC_VALID <= ERROR_RHA_ASYNC_VALID_INTERNAL;
	-- If the RHAs get asynchronous we want to know
	ASYNC_CHECK : process (Clk,Reset)
	begin
		
		if Reset = '1' then
			ERROR_RHA_ASYNC_FIRST_INTERNAL <= '0';
			ERROR_RHA_ASYNC_VALID_INTERNAL <= '0';
			
		elsif rising_edge(Clk) then
			
			ERROR_RHA_ASYNC_FIRST_INTERNAL <= ERROR_RHA_ASYNC_FIRST_INTERNAL;
			ERROR_RHA_ASYNC_VALID_INTERNAL <= ERROR_RHA_ASYNC_VALID_INTERNAL;
			
			-- VALID
			-- If GATED_VALID is completely zero, then everything is fine
			if GATED_VALID = "00000000" then
				null;
			-- If GATED_VALID is not zero but is equal to the selection mask then everything is fine
			elsif GATED_VALID = RHA_SELECTION_MASK then
				null;
			-- If something else happens that is not fine 
			else
				ERROR_RHA_ASYNC_VALID_INTERNAL <= '1';
			end if;
			
			-- FIRST
			-- If GATED_FIRST is completely zero, then everything is fine
			if GATED_FIRST = "00000000" then
				null;
			-- If GATED_FIRST is not zero but is equal to the selection mask then everything is fine
			elsif GATED_FIRST = RHA_SELECTION_MASK then
				null;
			-- If something else happens that is not fine 
			else
				ERROR_RHA_ASYNC_FIRST_INTERNAL <= '1';
			end if;
			
		end if;
	end process ASYNC_CHECK;	

end architecture Behavioral;





